/**
* GMyth Library
*
* @file gmyth/gmyth_util.c
* 
* @brief <p> This component provides utility functions.
* 
* Copyright (C) 2006 INdT - Instituto Nokia de Tecnologia.
* @author Hallyson Luiz de Morais Melo <hallyson.melo@indt.org.br>
*
*//*
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
 
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#define _XOPEN_SOURCE
#define _XOPEN_SOURCE_EXTENDED

#include <glib.h>
#include <glib/gprintf.h>
#include <time.h>
#include <sys/time.h>

#include "gmyth.h"

static GStaticMutex mutex = G_STATIC_MUTEX_INIT;

/** Converts a time_t struct in a GString at ISO standard format 
 * (e.g. 2006-07-20T09:56:41).
 * 
 * The returned GString memory should be deallocated from 
 * the calling function.
 *
 * @param time_value the time value to be converted
 * @return GString* the converted isoformat string 
 */
GString*
gmyth_util_time_to_isoformat (time_t time_value)
{
	struct tm tm_time;
	GString *result;
	
	g_static_mutex_lock ( &mutex );
	
	if (localtime_r(&time_value, &tm_time) == NULL) {
		g_static_mutex_unlock ( &mutex );
		g_warning ("gmyth_util_time_to_isoformat convertion error!\n");
        return NULL;
	}
	
	result = g_string_sized_new(20);
	g_string_printf(result, "%04d-%02d-%02dT%02d:%02d:%02d",
		tm_time.tm_year + 1900, tm_time.tm_mon + 1, tm_time.tm_mday,
		tm_time.tm_hour, tm_time.tm_min, tm_time.tm_sec);
		
	gmyth_debug( "Result (ISO 8601) = %s", result->str );
		
	g_static_mutex_unlock ( &mutex );
	
	return result;
	
	
}

/** Converts a time_t struct in a GString at ISO standard format 
 * (e.g. 2006-07-20T09:56:41).
 * 
 * The returned GString memory should be deallocated from 
 * the calling function.
 *
 * @param time_value the GTimeValue to be converted
 * @return GString* the converted isoformat string 
 */
static gchar*
gmyth_util_time_to_isoformat_from_time_val_fmt ( const gchar *fmt_string, GTimeVal* time)
{
	
	gchar *result = g_time_val_to_iso8601( time );
	gmyth_debug ("GMythUtil: before transcoding, result is: %s!\n", result);
	
	struct tm* tm_timeval = g_new0( struct tm, 1 );
	
	g_return_val_if_fail( fmt_string != NULL, NULL );

  if ( NULL == time ) { 
		gmyth_debug ("GMythUtil: time_to_isoformat_from_time converter error (timeval == NULL)!\n");
		return NULL;
	}
	
	g_static_mutex_lock ( &mutex );
	
	strptime( result, "%Y-%m-%dT%H:%M:%SZ", tm_timeval ); 
	
	//g_date_set_time_val( date, time );
	
	//g_date_strftime( result, 21, fmt_string, date );
	strftime( result, strlen(result), fmt_string, tm_timeval );
	
	g_static_mutex_unlock ( &mutex );
	
	gmyth_debug( "Result (ISO 8601) = %s", result  );
	
	//if ( date != NULL )
	//	g_date_free( date );
	
	return result;	
		
}

/** Converts a time_t struct in a GString at ISO standard format 
 * (e.g. 2006-07-20T09:56:41).
 * 
 * The returned GString memory should be deallocated from 
 * the calling function.
 *
 * @param time_value the GTimeValue to be converted
 * @return GString* the converted isoformat string 
 */
gchar*
gmyth_util_time_to_isoformat_from_time_val ( GTimeVal* time )
{
	gchar *result = gmyth_util_time_to_isoformat_from_time_val_fmt( "%Y-%m-%d %T", time );
	//result[10] = ' ';
	//result[ strlen(result) - 1] = '\0';
	
	return result;	
}

/** Converts a time_t struct in a GString at ISO standard format 
 * (e.g. 2006-07-20T09:56:41).
 * 
 * The returned GString memory should be deallocated from 
 * the calling function.
 *
 * @param time_value the GTimeValue to be converted
 * @return GString* the converted isoformat string 
 */
gchar*
gmyth_util_time_to_string_only_date ( GTimeVal* time )
{
	gchar *result = gmyth_util_time_to_isoformat_from_time_val_fmt( "%y-%m-%d", time );
	result[10] = ' ';
	result[ strlen(result) - 1] = '\0';
	return result;
}

/** Converts a time_t struct in a GString at ISO standard format 
 * (e.g. 2006-07-20T09:56:41).
 * 
 * The returned GString memory should be deallocated from 
 * the calling function.
 *
 * @param time_value the GTimeValue to be converted
 * @return GString* the converted isoformat string 
 */
gchar*
gmyth_util_time_to_string_only_time ( GTimeVal* time )
{
	gchar *result = gmyth_util_time_to_isoformat_from_time_val_fmt( "%T", time );
	result[10] = ' ';
	result[ strlen(result) - 1] = '\0';
	return result;
}

/** Converts a time_t struct in a GString to the following 
 * format (e.g. 2006-07-20 09:56:41).
 * 
 * The returned GString memory should be deallocated from 
 * the calling function.
 *
 * @param time_value the time value to be converted
 * @return GString* the converted string 
 */
GString*
gmyth_util_time_to_string (time_t time_value)
{
	GString *result = gmyth_util_time_to_isoformat (time_value);
	result->str[10] = ' ';	

	return result;
}

/** Converts a time_t struct in a GString to the following 
 * format (e.g. 2006-07-20 09:56:41).
 * 
 * The returned GString memory should be deallocated from 
 * the calling function.
 *
 * @param time_value the time value to be converted
 * @return GString* the converted string 
 */
gchar*
gmyth_util_time_to_string_from_time_val (GTimeVal *time_val)
{
	gchar *result = gmyth_util_time_to_isoformat_from_time_val (time_val);

	return result;
}

/** Converts a GString in the following format 
 * (e.g. 2006-07-20 09:56:41) to a time_t struct.
 * 
 * @param time_str the string to be converted
 * @return time_t the time converted value
 */
time_t
gmyth_util_string_to_time (GString* time_str)
{
	gint year, month, day, hour, min, sec;
	GDate *date = g_date_new();
    
	gmyth_debug( "[%s] time_str = %s. [%s]\n", __FUNCTION__, time_str != NULL ? 
					time_str->str : "[time string is NULL!]", g_strdelimit( time_str->str, " ", 'T' ) );


	//if (sscanf (time_str->str, "%04d-%02d-%02d %02d:%02d:%02d",
	//		&year, &month, &day, &hour, &min, &sec) < 3) {				
	//g_date_set_parse( date, time_str->str );
  if ( NULL == date ) { 
		g_warning ("GMythUtil: isoformat_to_time converter error!\n");
		return 0;
	}
	
	g_static_mutex_lock ( &mutex );
	//GDate *date = g_date_new_dmy( day, month - 1, year - 1900 );
	
	struct tm* tm_time = g_malloc0( sizeof(struct tm) );
/*		
		tm_time->tm_year = year - 1900;
		tm_time->tm_mon = month - 1;
		tm_time->tm_mday = day;
		tm_time->tm_hour = hour;
		tm_time->tm_min = min;
		tm_time->tm_sec = sec;
	*/
	
	GTimeVal *time = g_new0( GTimeVal, 1 ); 
	
	if ( !g_time_val_from_iso8601( g_strdelimit( time_str->str, " ", 'T' ), time ) )
	{
		gmyth_debug( "ERROR! Problems converting to GTimeVal == %s\n", g_time_val_to_iso8601( time ) );
	} 
	
	gmyth_debug( "Converted to GTimeVal == %s\n", g_time_val_to_iso8601( time ) );
	
	g_date_set_time_val( date, time );
	
	g_date_to_struct_tm( date, tm_time ); 
	
	g_static_mutex_unlock ( &mutex );
	
	return mktime( tm_time );
}

/** Converts a GString in the following format 
 * (e.g. 2006-07-20 09:56:41) to a time_t struct.
 * 
 * @param time_str the string to be converted
 * @return time_t the time converted value
 */
GDate *
gmyth_util_time_val_to_date (GTimeVal* time)
{
	GDate *date = g_date_new();
    
  if ( NULL == date ) { 
		g_warning ("GMythUtil: GDate *gmyth_util_time_val_to_date (GTimeVal* time) - converter error!\n");
		return 0;
	}
	
	g_date_set_time_val( date, time );
	
	gmyth_debug( "Converted from GTimeVal == %s to GDate\n", g_time_val_to_iso8601( time ) );
	
	return date;
}

/** Converts a GString in the following format 
 * (e.g. 2006-07-20 09:56:41) to a time_t struct.
 * 
 * @param time_str the string to be converted
 * @return time_t the time converted value
 */
GTimeVal*
gmyth_util_string_to_time_val (gchar* time_str)
{
	GTimeVal *time = g_new0( GTimeVal, 1 ); 
    
	gmyth_debug( "[%s] time_str = %s. [%s]\n", __FUNCTION__, time_str != NULL ? 
					time_str : "[time string is NULL!]", g_strdelimit( time_str, " ", 'T' ) );

  if ( NULL == time ) { 
		g_warning ("GMythUtil: isoformat_to_time converter error!\n");
		return NULL;
	}
	
	g_static_mutex_lock ( &mutex );
	
	if ( !g_time_val_from_iso8601( g_strdelimit( time_str, " ", 'T' ), time ) )
	{
		gmyth_debug( "ERROR! Problems converting to GTimeVal == %s\n", g_strdelimit( time_str, " ", 'T' ) );
	} 
	
	gmyth_debug( "Converted to GTimeVal == %s\n", g_time_val_to_iso8601( time ) );
	
	g_static_mutex_unlock ( &mutex );
	
	return time;
}

/** Decodes a long long variable from the string list
 * format of the myhtprotocol.
 * 
 * @param strlist the string list of mythprotocol values
 * @param offset  the list node offset of the long long variable
 * @return gint64  the long long converted value
 */
gint64
gmyth_util_decode_long_long(GMythStringList *strlist, guint offset) 
{

	gint64 ret_value = 0LL;

	g_return_val_if_fail( strlist != NULL, ret_value );

	if ( offset > gmyth_string_list_length( strlist ))
		g_printerr( "[%s] Offset is greater than the Stringlist (offset = %d)!\n", 
                    __FUNCTION__, offset );
    
	g_return_val_if_fail( offset < gmyth_string_list_length( strlist ), ret_value );

	gint l1 = gmyth_string_list_get_int( strlist, offset );
	gint l2 = gmyth_string_list_get_int( strlist, offset + 1 );

	ret_value = (l2 /*& 0xffffffffLL*/) | ( (gint64)l1 << 32 );

	return ret_value;

}

gboolean
gmyth_util_file_exists (GMythBackendInfo *backend_info, const gchar* filename)
{
    GMythSocket *socket;
    gboolean res;
    
    socket = gmyth_socket_new ();
    res = gmyth_socket_connect_to_backend (socket, backend_info->hostname,
		    backend_info->port, TRUE);

    if (res == TRUE) {
        GMythStringList *slist;
        GMythProgramInfo *program;

        program = g_new0 (GMythProgramInfo, 1);
        program->pathname = g_string_new (filename);

        slist = gmyth_string_list_new ();
        gmyth_string_list_append_char_array (slist, "QUERY_CHECKFILE");

        gmyth_program_info_to_string_list (program, slist);

        gmyth_socket_sendreceive_stringlist (socket, slist);

        res = (gmyth_string_list_get_int (slist, 0) == 1);
    
        // fixme: we should do this in a program_info_free() function
        g_string_free (program->pathname, TRUE);
        g_free (program);

        g_object_unref (slist);

        gmyth_socket_close_connection (socket);
    }
    g_object_unref (socket);
    return res;    
}

